package Google::API::Resource;

use strict;
use warnings;

use base 'Google::API::Base';

1;
__END__

=encoding utf-8

=for stopwords

=head1 NAME

Google::API::Resource - A implementation of resources part in Discovery Document Resource

=head1 SYNOPSIS

  use Google::API::Resource;

=head1 DESCRIPTION

Google::API::Resource is an implementation of resources part in Discovery Document Resource

=head1 AUTHOR

Takatsugu Shigeta E<lt>shigeta@cpan.orgE<gt>

=head1 COPYRIGHT

Copyright 2011- Takatsugu Shigeta

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 SEE ALSO

=cut
